#!/usr/bin/env python

from __future__ import division
import numpy as np
from scipy.io import wavfile
import scipy.signal as sig

DATA_LENGTH = 65  # seconds
SAMPLE_RATE = 24000*128  # Hz
DURATION = DATA_LENGTH*SAMPLE_RATE
OUTPUT_SAMPLE_RATE = 48000 # Hz

def secs_to_samples(seconds):
  return int(round(seconds * SAMPLE_RATE))

def ternary(ch0, ch1):
  # Assuming active-low, ch0 is positive.
  if ch0 == '0' and ch1 == '1':
    return 1.
  if ch0 == '1' and ch1 == '0':
    return -1.
  return 0.

def parse(f):
  # Because I'm too lazy to import csv.
  # Skip the header and the first datapoint.
  next(f); next(f)
  for line in f:
    time_str, ch0, ch1 = line.strip().split(', ')
    time = float(time_str)
    yield time, ch0, ch1


# Expand the data into samples.
data = np.empty(DURATION)
datafile = parse(open('abc.csv'))

time_offset, ch0, ch1 = next(datafile)
last_sample = ternary(ch0, ch1)
last_t = 0
for seconds, ch0, ch1 in datafile:
  if seconds - time_offset > DURATION:
    data[last_t:DURATION] = last_sample
    break
  t = secs_to_samples(seconds-time_offset)
  data[last_t:t] = last_sample
  last_t = t
  last_sample = ternary(ch0, ch1)
else:
  data[last_t:DURATION] = last_sample

# Decimate to a playable sample rate. The 'decimate' function also low-pass
# filters the signal to the nyquist rate (1/2 the sample rate). This simulates
# the output filter on a Class D amplifier.
# While the filter used in 'decimate' is not the best for audio, but neither
# would the one used in the Hit Clip devices. It's good enough, and it's fast.
data = sig.decimate(data, int(SAMPLE_RATE/OUTPUT_SAMPLE_RATE))
# Convert to 16-bit signed integer values and save it as a WAV file.
data = (np.clip(data, -1., 1.) * (2**15-1)).astype(np.int16)
wavfile.write('abc.wav', OUTPUT_SAMPLE_RATE, data)
